@@ -1,10 +1,11 @@
 from datetime import date
 
+from django.contrib.contenttypes.fields import GenericRelation
 from django.db import models
 from django.forms import ValidationError
 from django.urls import reverse
 
-from netbox.models import NetBoxModel
+from netbox.models import NetBoxModel, NestedGroupModel
 from .choices import HardwareKindChoices, AssetStatusChoices
 from .utils import asset_clear_old_hw, get_prechange_field, get_plugin_setting, get_status_for
 
@@ -174,6 +175,10 @@ class Asset(NetBoxModel):
         blank=True
     )
 
+    images = GenericRelation(
+        to='extras.ImageAttachment'
+    )
+
     clone_fields = [
         'status', 'device_type', 'module_type', 'inventoryitem_type', 'tenant', 'contact',
         'storage_location', 'owner', 'purchase', 'warranty_start', 'warranty_end', 'comments'
@@ -200,6 +205,33 @@ class Asset(NetBoxModel):
     def hardware(self):
         return self.device or self.module or self.inventoryitem or None
 
+    @property
+    def installed_site(self):
+        device = self.installed_device
+        if device:
+            return device.site
+
+    @property
+    def installed_location(self):
+        device = self.installed_device
+        if device:
+            return device.location
+
+    @property
+    def installed_rack(self):
+        device = self.installed_device
+        if device:
+            return device.rack
+
+    @property
+    def installed_device(self):
+        if self.kind == 'device':
+            return self.device
+        elif self.hardware:
+            return self.hardware.device
+        else:
+            return None
+
     @property
     def warranty_remaining(self):
         """
@@ -358,6 +390,9 @@ class Supplier(NetBoxModel):
         max_length=200,
         blank=True
     )
+    contacts = GenericRelation(
+        to='tenancy.ContactAssignment'
+    )
     comments = models.TextField(
         blank=True
     )
@@ -454,6 +489,9 @@ class InventoryItemType(NetBoxModel):
     comments = models.TextField(
         blank=True
     )
+    images = GenericRelation(
+        to='extras.ImageAttachment'
+    )
 
     clone_fields = [
         'manufacturer',
@@ -473,25 +511,33 @@ class InventoryItemType(NetBoxModel):
         return reverse('plugins:netbox_inventory:inventoryitemtype', args=[self.pk])
 
 
-class InventoryItemGroup(NetBoxModel):
+class InventoryItemGroup(NestedGroupModel):
     """
-    Inventory Item Groups are groups fo simmilar InventoryItemTypes.
+    Inventory Item Groups are groups of simmilar InventoryItemTypes.
     This allows you to, for example, have one Group for all your 10G-LR SFP
     pluggables, from different manufacturers/with different part numbers.
+    Inventory Item Groups can be nested.
     """
-    name = models.CharField(
-        max_length=100,
-        unique=True,
-    )
+    slug = None # remove field that is defined on NestedGroupModel
+
     comments = models.TextField(
         blank=True
     )
 
     class Meta:
         ordering = ['name']
-
-    def __str__(self):
-        return self.name
+        constraints = (
+            models.UniqueConstraint(
+                fields=('parent', 'name'),
+                name='%(app_label)s_%(class)s_parent_name'
+            ),
+            models.UniqueConstraint(
+                fields=('name',),
+                name='%(app_label)s_%(class)s_name',
+                condition=models.Q(parent__isnull=True),
+                violation_error_message="A top-level group with this name already exists."
+            ),
+        )
 
     def get_absolute_url(self):
         return reverse('plugins:netbox_inventory:inventoryitemgroup', args=[self.pk])
